from distutils.core import setup

setup(
    name='bottle-jwt',
    version='0.0.1',
    packages=['bottle_jwt'],
    url='',
    license='GLPv3',
    author='Papavassiliou Vassilis',
    author_email='vpapavasil@gmail.com',
    description='JWT Auth plugin for bottle.py applications'
)
